/* NAME:
 *	hash - simple hash routine
 *
 * SYNOPSIS:
 *	unsigned long HashM(const char *"s", int "n", int "m")
 *	unsigned long HashN(const char *"s", int "n")
 *	unsigned long Hash(const char *"s")
 *
 * DESCRIPTION:
 *      Provide a convenient numeric representation of a string.
 *	HashM() does all the work.  If "n" is less than 0, it uses the
 *	length of "s", the mask "m" is applied to each char in "s" up
 *	to "n" before hashing.  A mask of ~0x20 will cause the hash
 *	values to be case insensitive (for ascii anyway).
 *
 *	HashN() simply calls HashM(s, n, 0);
 *
 *	Hash() simply calls HashM(s, -1, 0);
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */

#ifndef lint
static char RCSid[] = "$Id: hash.c,v 1.4 1998/09/13 02:49:43 sjg Exp $";
#endif

#include <ctype.h>
#include <sys/cdefs.h>

unsigned long
HashM(s, n, m)
	const char *s;
	int n;
	int m;
{
	register int i, c;
	unsigned long val = 0;

	if (s && *s) {
		if (n < 0)
			n = strlen(s);
		for (i = 0; s[i] && i < n; ++i) {
			c = s[i];
			if (m)
				c &= m;
			val += c * (c << (i % 30));
		}
	}
	return val;
}


unsigned long
HashN(s, n)
	const char *s;
	int n;
{
	return HashM(s, n, 0);
}

unsigned long
Hash(s)
	const char *s;
{
	return HashM(s, -1, 0);
}

#ifdef MAIN
int
main(argc, argv)
	int argc;
	char *argv[];
{
	int m = 0;
	int n = -1;

	if (argc > 2)
		n = atoi(argv[2]);
	if (argc > 3)
		m = atoi(argv[3]);
	if (argc > 1)
		printf("%s->%#lx\n", argv[1], HashM(argv[1], n, m));
	exit(0);
}
#endif
